#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED   0
#define VENDOR_RQ_GET_TEMP  1
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}
uint16_t read_adc()
{
//	unsigned char i,ch;
//	uint16_t TEMP;
	DDRB=0xFF;
	ACSR=0x80;
	ADCSRB=0x00;
       	ADMUX=0xC0; // ADC Voltage Reference: Int., cap. on AREF
        ADCSRA=0x87; // ADC Clock frequency: 62.500 kHz
       	ADMUX|= 5; // 3 bcz i am using adc channel 3
       // Delay needed for the stabilization of the ADC input voltage
        _delay_us(100);
      //	ADCSRA|=0x40; //Initialize ADC in single conversion mode
	ADCSRA|=(1<<ADSC);
        while ((ADCSRA & (1<<ADIF))==0); // Wait for the AD conversion to complete
	ADCSRA|=(1<<ADIF);
/*	while(!(ADCSRA&0x10));
	ADCSRA|=0x20;*/
	return(ADC);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[16])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_TEMP)
    {
	uint16_t x;
	x = read_adc();
	dataBuffer[0] = x/256;
	dataBuffer[1] = x%256;
//	usbMsgPtr = x;
	usbMsgPtr = dataBuffer;
	return 2;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = 0b00000111;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0
    PORTC |= (1 << 3);   // Enable pull-up at PC3
    DDRD  = 0b00001000;
    PORTD = 0b00000000;
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    uint16_t i;
    uint16_t y,TEMP=0;
    for(;;){                /* main event loop */
        usbPoll();
//	y = read_adc();
	
/*        for(i=0;i<100;i++)
      	{
              TEMP=TEMP+y;
       	}
        TEMP/=400.0;
	TEMP+=10;
	if(y>0&&y<=10) out_c(5,1);
	else if(y>10&&y<20) out_c(4,1);
	else if(y>20&&y<30) out_c(2,1);
	else if(y>30&&y<50) out_c(0,1);
	else if(y>50) out_c(1,1);*/
    }
    return 0;
}
/* ------------------------------------------------------------------------- */
